import pymel.all as pm
from MARS.MarsRigComponent import MarsRigComponent
import MARS.MarsUtilities as mu
reload(mu)

class FlexibleEyelid(MarsRigComponent):
    '''
    #MANDATORY VARIABLES
        side = Use the string center, outer, or inner.
        limb = Use a string to describe what body part of the rig this applies to.
        upper_lids = Use a list of the joints for the upper lip from the character's inner to outer.
                     Does not include corner joints.
        lower_lids = Use a list of the joints for the lower lip from the character's inner to outer.
                     Does not include corner joints.
        inner_corner = Use a string for the name of the inner corner bone.
        outer_corner = Use a string for the name of the outer corner bone.
        
    #OPTIONAL VARIABLES
        nurb = Requires the string of a nurbSurface. This surface should map out where the teeth are.
        secondary_anims = Defaults to True. Creates a second layer of anims for added fine movement.
        jaw = Default to None. Requires a string of a jaw bone. Connects the lower lip to the jaw bone.
    
    #SPECIALTY VARIABLES
        connect_to = Default to None. Requires a class listed as a variable to parent the current node to.
                     Needs the 'area' variable to work
        area = Default to None. Uses 'start', 'end', or a specific bone string connected to the 
               variable in the 'connected_to' variable.
               
        rig = Default to None. Requires the class of a CharacterRig node.
              Adds node to the component_grp of the CharacterRig node.
              
        node = Default to None. Used for the Marking Menu. Used for reacquiring the class functionality.
    '''
    def __init__(self, side, limb, upper_lids, lower_lids, inner_corner, outer_corner, eye_joint, scale_setting = (1,1,1),
                 secondary_anims = True, node = None, connect_to = None, area = None, rig = None):
        if node != None: self.node = node
        else: self.node = self.exist_check('FlexibleEyelid',upper_lids[0],'upper_lid_bones')
         
        if self.node == None:
            #Run Function
            MarsRigComponent.__init__(self,'FlexibleEyelid', side, limb, start = '', end = '')
            
            #Variables
            directions = ['upper','lower']
            corners = ['inner','outer']
            lid = {}
            controls = []
            IK_jnts = []
            IK_handles = []
            locators = []
            anims = {}
            all_anims = []
            anims['primary'] = []
            anims['secondary'] = []
            anims['corners'] = []
            name = side + '_' + limb
            smooth_nurbs = {}
            eye_joint = pm.PyNode(eye_joint)
            lid['outer'] = [pm.PyNode(outer_corner)]
            lid['inner'] = [pm.PyNode(inner_corner)]
            lid['upper'] = [pm.PyNode(x) for x in upper_lids]
            lid['lower'] = [pm.PyNode(x) for x in lower_lids]
            lid['full_upper'] = lid['inner'] + lid['upper'] + lid['outer']
            lid['full_lower'] = lid['inner'] + lid['lower'] + lid['outer']
            bones = lid['inner'] + lid['upper'] + lid['outer'] + lid['lower']

            #Group Setup
            extra_grps = ['primary_anim','secondary_anim','control','aim_grp','nurbs','scale']
            component_grp, hidden_grp, anim_grp, other_grps = self.component_group_setup(name, other_grps = extra_grps)
            prim_grp, sec_grp, control_grp, aim_grp, nurb_grp, scale_grp = other_grps
            
            pm.parent(prim_grp, sec_grp, anim_grp)
            pm.parent(scale_grp, nurb_grp, aim_grp, control_grp,hidden_grp)
            aim_grp.inheritsTransform.set(0)
            mu.aligner([scale_grp],eye_joint,rotation = False)
            scale_grp.scale.set(scale_setting)
            
            #Aim Setup
            world_up_vector = pm.spaceLocator(p = (0,0,0))
            mu.aligner(world_up_vector,eye_joint,rotation = False)
            world_up_vector.ty.set(world_up_vector.ty.get() * 1.1)
            pm.parent(world_up_vector, hidden_grp)
            
            ##Anim Setup
            for c in corners:
                anims[c], c_null = mu.create_anims(lid[c],'{0}_{1}'.format(name,c),end = False,type = 'sphere')
                self.orient_anims([c_null],eye_joint)
                pm.parent(c_null, prim_grp)
                anims['corners'] += [anims[c]]
            
            for i, d in enumerate(directions):
                n = '{0}_{1}'.format(name,d)
                control_bones = lid['full_' + d]
                if d == 'lower': control_bones = lid['lower']
                anims[d], nulls, nurbs = self.create_lid_anims(n,lid['full_' + d])
                anims['main_' + d], main_null = mu.create_anims(anims[d][2],n + '_eyelid',end = False)
                self.orient_anims([main_null] + nulls,eye_joint)
                smooth_nurbs[d], unsmoothed_nurb, nurb_wire = nurbs
                lid_controls, control_locs, aim_locs = self.control_setup(n,control_bones,eye_joint,unsmoothed_nurb,scale_grp,world_up_vector)
                pm.parent(nurbs,nurb_grp)
                pm.parent(nulls,anims['main_' + d])
                pm.parent(main_null,prim_grp)
                pm.parent(lid_controls,control_grp)
                pm.parent(aim_locs,aim_grp)
                anims['primary'] += [anims[corners[i]]] + anims[d]
                all_anims += [anims['main_' + d]]
                locators += control_locs
                controls += lid_controls
            all_anims += anims['primary']
            
            #Skin Jnts
            full_upper_anims = [anims['inner']] + anims['upper'] + [anims['outer']] 
            full_lower_anims = [anims['inner']] + anims['lower'] + [anims['outer']] 
            upper_skin_jnts = mu.duplicate_chain(anims['upper'],name + '_upper','skin_joint',separate = True)
            lower_skin_jnts = mu.duplicate_chain(anims['lower'],name + '_lower','skin_joint',separate = True)
            inner_skin_jnt = mu.duplicate_chain(anims['inner'],name + '_right','skin_joint',separate = True)[0]
            outer_skin_jnt = mu.duplicate_chain(anims['outer'],name + '_left','skin_joint',separate = True)[0]
            full_upper_lip_skin = [inner_skin_jnt] + upper_skin_jnts + [outer_skin_jnt]
            full_lower_lip_skin = [inner_skin_jnt] + lower_skin_jnts + [outer_skin_jnt]
            skin_jnts = [inner_skin_jnt] + upper_skin_jnts + [outer_skin_jnt] + lower_skin_jnts
            pm.skinCluster(full_upper_lip_skin,smooth_nurbs['upper'],n = smooth_nurbs['upper'] + '_skin',tsb = True, mi = 2, sm = 1, nw = 1)
            pm.skinCluster(full_lower_lip_skin,smooth_nurbs['lower'],n = smooth_nurbs['lower'] + '_skin',tsb = True, mi = 2, sm = 1, nw = 1)
            [pm.parentConstraint(anims['primary'][i],x,mo = True) for i, x in enumerate(skin_jnts)]
            pm.parent(skin_jnts,hidden_grp)
            
            #Primary Anim Setup
            pm.pointConstraint(anims['upper'][1],anims['inner'],anims['upper'][0].getParent(),mo = True)
            pm.pointConstraint(anims['upper'][-2],anims['outer'],anims['upper'][-1].getParent(),mo = True)
            pm.pointConstraint(anims['lower'][1],anims['inner'],anims['lower'][0].getParent(),mo = True)
            pm.pointConstraint(anims['lower'][-2],anims['outer'],anims['lower'][-1].getParent(),mo = True)
            
            #Secondary Anim and Constraint Setup
            if secondary_anims == True: 
                anims['secondary'], sec_nulls = mu.create_anims(bones, name, suffix = 'secondary_anim', separate = True, end = False)
                [pm.parentConstraint(sec, controls[i], mo = True) for i, sec in enumerate(anims['secondary'])]
                [pm.pointConstraint(loc, sec_nulls[i], mo = True) for i, loc in enumerate(locators)]
                pm.parent(sec_nulls, sec_grp)
                all_anims += anims['secondary']
            else: [pm.parentConstraint(loc, controls[i], mo = True) for i, loc in enumerate(locators)]
            
            #Nurb Setup
            mid_nurb = self.make_middle_nurb(name, lid['full_upper'],lid['full_lower'])
            s_upper_nurb, s_lower_nurb = self.middle_blend_setup(name,anims['main_upper'], anims['main_lower'], smooth_nurbs['upper'], smooth_nurbs['lower'],mid_nurb)
            pm.parent(mid_nurb, nurb_grp)
            
            #Lock And Hide Attrs
            mu.lock_and_hide_attrs(all_anims,['scale','radius','visibility'])
            mu.lock_and_hide_attrs(other_grps,'all')
            mu.lock_and_hide_attrs(hidden_grp,'all')
            if secondary_anims == False: pm.delete(sec_grp)
            
            #Connections
            mu.con_link(bones,self.node,'bones')
            mu.con_link(controls,self.node,'controls')
            mu.con_link(all_anims,self.node,'anims')
            mu.con_link([anims['main_upper'],anims['main_lower']],self.node,'main_anims')
            mu.con_link(anims['primary'],self.node,'primary_anims')
            mu.con_link(anims['corners'],self.node,'corner_anims')
            mu.con_link(anims['upper'],self.node,'upper_anims')
            mu.con_link(anims['lower'],self.node,'lower_anims')
            mu.con_link(skin_jnts,self.node,'skin_joints')
            if secondary_anims == True: mu.con_link(anims['secondary'],self.node,'secondary_anims')
            mu.con_link([smooth_nurbs['upper'],smooth_nurbs['lower']],self.node,'skinned_curves')
            mu.con_link(lid['upper'],self.node,'upper_lid_bones')
            mu.con_link(lid['lower'],self.node,'lower_lid_bones')
            mu.con_link(lid['inner'] + lid['outer'],self.node,'corner_bones')
            mu.con_link(component_grp,self.node,'component_grp')
            mu.con_link(hidden_grp,self.node,'hidden_grp')
            mu.con_link(anim_grp,self.node,'anim_grp')
            mu.con_link(prim_grp,self.node,'primary_anim_grp')
            
            ##Finish
            if connect_to != None and area != None: self.connect_component_to(connect_to,area)
            if rig != None: rig.add_to_rig(self)
            pm.select(cl = True)
    
    def orient_anims(self,anim_nulls,eye_joint):
        for an in anim_nulls:
            pm.parent(an,eye_joint)
            an.jointOrient.set(0,0,0)
            pm.parent(an,w = True)
    
    def make_middle_nurb(self, name, first_chain, second_chain):
        amnt = len(first_chain)
        in_b = first_chain[0]
        out_b = first_chain[-1]
        locators = [pm.spaceLocator(p = (0,0,0)) for i in range(amnt)]
        start = locators[0]
        end = locators[-1]
        if amnt % 2 == 0:
            center = pm.group(n = 'temp_center',em = True)
            inside = locators[1:(amnt / 2) + 1]
            outside = locators[(amnt / 2) + 1:-1]
        else:
            center = locators[(amnt - 1)/ 2]
            inside = locators[1:(amnt / 2)]
            outside = locators[(amnt / 2) + 1:-1]
        complete_inside = [start] + inside + [center]
        complete_outside = [center] + outside + [end]
        
        mu.aligner(start,in_b,rotation = False)
        mu.aligner(end,out_b,rotation = False)
        pm.pointConstraint(start,end,center,mo = False)
        for i, x in enumerate(inside): pm.pointConstraint(start,complete_inside[i + 2],x,mo = False)
        for i, x in enumerate(outside): pm.pointConstraint(end,complete_outside[i],x,mo = False)
        
        cv = mu.make_spline(name + 'center',locators,spans = amnt - 1,smooth = True)
        pm.delete(locators)
        if len(locators) % 2 == 0: pm.delete(temp_center)
        
        return cv

    def middle_blend_setup(self,name,top_anim,bottom_anim,upper_nurb,lower_nurb,mid_nurb):
        start_upper_nurb = pm.duplicate(upper_nurb,n = upper_nurb + '_start')[0]
        start_lower_nurb = pm.duplicate(lower_nurb,n = lower_nurb + '_start')[0]
        start_upper_nurb.inheritsTransform.set(1)
        start_lower_nurb.inheritsTransform.set(1)
        new_nurbs = [start_upper_nurb,start_lower_nurb]
        
        upper_blend = pm.blendShape(mid_nurb,upper_nurb,n = upper_nurb + '_blend',tc = True)[0]
        lower_blend = pm.blendShape(mid_nurb,lower_nurb,n = lower_nurb + '_blend',tc = True)[0]
        middle_blend = pm.blendShape(start_upper_nurb, start_lower_nurb, mid_nurb, n = mid_nurb + '_blend',tc = True)[0]
        blends = [upper_blend,lower_blend]
        
        #Attribute Setup
        for i, anim in enumerate([top_anim,bottom_anim]):
            anim.addAttr('blink',at = 'double', dv = 0, min = 0, max = 1, k = True)
            anim.blink >> blends[i].attr(mid_nurb)
        top_anim.addAttr('blink_height',at = 'double', dv = 0.5 ,min = 0,max = 1,k = True)
        
        rev = mu.create_node('reverse',n = name + '_blink_rev')
        md = mu.create_node('multiplyDivide',n = name + 'blend_md')
        sub_md = mu.create_node('multiplyDivide',n = name + 'sub_value')
        cond = mu.create_node('condition',n = name + '_blink_condition')
        md.input2.set(2,2,2)
        sub_md.input1X.set(0.5)
        cond.secondTerm.set(0.5)
        cond.operation.set(3)
        cond.colorIfFalse.set(0,0,0)
        top_anim.blink_height >> rev.inputX
        top_anim.blink_height >> cond.firstTerm
        md.outputX >> cond.colorIfTrueR
        md.outputY >> cond.colorIfFalseG
        cond.outColorR >> middle_blend.attr(start_upper_nurb)
        cond.outColorG >> middle_blend.attr(start_lower_nurb)
        for x in ['upper','lower']:
            pma = pm.createNode('plusMinusAverage')
            pma.operation.set(2)
            if x == 'upper':
                top_anim.blink_height >> pma.input1D[0]
                pma.output1D >> md.input1X
            else:
                pma.output1D >> md.input1Y
                rev.outputX >> pma.input1D[0]
            sub_md.outputX >> pma.input1D[1]
        
        return start_upper_nurb, start_lower_nurb
    
    def control_setup(self,name,bones,eye_joint,nurb,scale_grp,wuo_loc):
        #Variables
        locators = []
        aim_locators = []
        controls = mu.duplicate_chain(bones,name,'control_joint',separate = True)
        ex, x = 1.00, -1.00
        if 'upper' in name: ex, x = 0, 1.00
        prct = 1.00/(len(bones) - x)
        
        #Setup
        for i, c in enumerate(controls):
            loc = pm.spaceLocator(n = c + '_locator', p = (0,0,0))
            aim_loc = pm.spaceLocator(n = c + '_aim',p = (0,0,0))
            grp = pm.group(n = c + '_locator_grp', em = True)
            
            mu.curve_constraint(nurb, aim_loc, (prct * (i + ex)))
            mu.aligner(grp, eye_joint)
            mu.aligner(loc,c)
            
            pm.parent(loc,grp)
            pm.parent(grp,scale_grp)
            pm.makeIdentity(grp,t = True,r = False,s = True,a = True)
            pm.parentConstraint(c, bones[i], mo = True)
            pm.aimConstraint(aim_loc, grp, mo = True,aim=(0,0,1), u=(0,1,0), wut='object', wuo= wuo_loc)
            locators += [loc]
            aim_locators += [aim_loc]
            
        return controls, locators, aim_locators
    
    def create_lid_anims(self,name,bones):
        areas = [0.1,0.25,0.5,0.75,0.9]
        unsmooth_nurb = mu.make_spline(name + '_bone',bones,spans = len(bones) - 1,smooth = False)
        unsmooth_nurb.inheritsTransform.set(0)
        anims, nulls, nurb = mu.create_spline_anims(name,'anim',bones,amnt = 5, spans = len(bones) - 1,spots = areas)
        wire_deform = pm.wire(unsmooth_nurb, gw=False,en=1.00,ce=0.00, li = 0.00, w = nurb)[0]
        wire_deform.dropoffDistance[0].set(1000)
        for n in nulls: n.jointOrient.set(0,0,0)
        ns = ['inner_corner','inner','middle','outer','outer_corner']
        for i, anim in enumerate(anims):
            anim.rename(name + '_' + ns[i] + '_anim')
            nulls[i].rename(name + '_' + ns[i] + '_null')
        nurb.rename(name + '_lid_nurb')
        return anims, nulls, [nurb, unsmooth_nurb,pm.PyNode(name + '_cvBaseWire')]
    
    def get_primary_anims(self):
        return pm.listConnections(self.node.primary_anims)
        
    def get_secondary_anims(self):
        if self.node.hasAttr('secondary_anims') == True: return pm.listConnections(self.node.secondary_anims)
        else: return []
        
    def setup_mirroring(self):
        anims = {}
        directions = ['upper','lower']
        if self.node.side.get() == 'left':
            primary_anim_grp = pm.listConnections(self.node.primary_anim_grp)[0]
            skin_joints = pm.listConnections(self.node.skin_joints)
            anim_nulls = [x.getParent() for x in self.get_all_anims()]
            primary_anims = pm.listConnections(self.node.primary_anims)
            primary_nulls = [x.getParent() for x in primary_anims]
            prim_pars = [x.getParent() for x in primary_nulls]
            main_upper_anim, main_lower_anim = pm.listConnections(self.node.main_anims)
            anims['upper'] = pm.listConnections(self.node.upper_anims)
            anims['lower'] = pm.listConnections(self.node.lower_anims)
            anims['corners'] = pm.listConnections(self.node.corner_anims)
            
            [mu.check_for_constraints(skj) for skj in skin_joints]
            mu.unlock_attrs(anim_nulls,['scale'])
            mu.unlock_attrs(primary_nulls,['translate','rotate'])
            pm.parent(primary_nulls, primary_anim_grp)
            [n.scale.set(-1,-1,-1) for n in anim_nulls]
            [pm.parent(primary_nulls[i],x) for i, x in enumerate(prim_pars)]
            mu.lock_and_hide_attrs(anim_nulls,['translate','rotate','scale'])
            [pm.parentConstraint(primary_anims[i],skj,mo = True) for i, skj in enumerate(skin_joints)]